<?php

namespace Drupal\Tests\domain_group\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\domain_group\Traits\GroupCreationTrait;
use Drupal\Tests\domain_group\Traits\InitializeGroupsTrait;

/**
 * Tests Domain Settings form.
 *
 * @group domain_group
 */
class DomainSettingsFormTest extends BrowserTestBase {

  use GroupCreationTrait;
  use InitializeGroupsTrait;

  /**
   * Will be removed when issue #3204455 on Domain Site Settings gets merged.
   *
   * See https://www.drupal.org/project/domain_site_settings/issues/3204455.
   *
   * @var bool
   *
   * @see \Drupal\Core\Config\Testing\ConfigSchemaChecker
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'block',
    'group',
    'domain',
    'domain_site_settings',
    'domain_group',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'classy';

  /**
   * Regular authenticated User for tests.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->adminUser = $this->drupalCreateUser([
      'access administration pages',
      'access group overview',
      'administer group',
      'bypass group access',
      'domain group settings',
    ]);

    // Setup the group types and test groups from the InitializeGroupsTrait.
    $this->initializeTestGroups();
  }

  /**
   * Test group domain settings form.
   */
  public function testDomainGroupForm() {
    $storage = \Drupal::entityTypeManager()->getStorage('domain');
    // No domains should exist.
    $this->domainTableIsEmpty();
    // Going to domain group settings form.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('group/' . $this->groupA1->id() . '/domain-settings');
    $this->assertSession()->pageTextContains($this->groupA1->label() . ' - Domain Settings');
    // Filling form.
    $edit = [
      'hostname' => 'group-a1.' . $this->baseHostname,
      'site_name' => $this->groupA1->label(),
      'site_mail' => 'group-' . $this->groupA1->id() . '@user.com',
    ];
    $this->submitForm($edit, 'Submit');
    // First save should fail.
    $this->assertSession()->pageTextContains('In order to enable this Organization domain, a Default one should be set.');
    // Create default domain programmatically.
    $domain = $storage->create();
    $domain->set('id', $storage->createMachineName($domain->getHostname()));
    $keys = [
      'id',
      'name',
      'hostname',
      'scheme',
      'status',
      'weight',
      'is_default',
    ];
    foreach ($keys as $key) {
      $property = $domain->get($key);
      $this->assertNotNull($property, 'Property loaded');
    }
    $domain->save();
    // Resubmit form.
    $this->submitForm($edit, 'Submit');
    // Did it save correctly?
    $this->assertSession()->pageTextContains('Changes saved');
    $ga1_domain = $storage->load('group_' . $this->groupA1->id());
    $this->assertNotEmpty($ga1_domain, 'Domain has been saved.');
    // Visiting group page.
    $this->drupalGet($ga1_domain->getPath());
    $this->assertSession()->pageTextContains($this->groupA1->label());
  }

}
